=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-08-09 19:30:31 +0000
+++ bzrlib/builtins.py	2007-08-09 20:30:00 +0000
@@ -3826,7 +3826,10 @@
                type=unicode),
         Option('output', short_name='o', help='Write directive to this file.',
                type=unicode),
+        Option('mail-to', help='Mail the request to this address',
+               type=unicode),
         'revision',
+        'message',
         RegistryOption.from_kwargs('format',
         'Use the specified output format.',
         **{'4': 'Bundle format 4, Merge Directive 2 (default)',
@@ -3835,15 +3838,17 @@
 
     def run(self, submit_branch=None, public_branch=None, no_bundle=False,
             no_patch=False, revision=None, remember=False, output=None,
-            format='4', **kwargs):
+            format='4', mail_to=None, message=None, **kwargs):
         return self._run(submit_branch, revision, public_branch, remember,
                          format, no_bundle, no_patch, output,
-                         kwargs.get('from', '.'))
+                         kwargs.get('from', '.'), mail_to, message)
 
     def _run(self, submit_branch, revision, public_branch, remember, format,
-             no_bundle, no_patch, output, from_,):
+             no_bundle, no_patch, output, from_, mail_to, message):
         from bzrlib.revision import ensure_null, NULL_REVISION
         if output is None:
+            if mail_to is None:
+                raise errors.BzrCommandError('No mail-to address specified')
             outfile = StringIO()
         elif output == '-':
             outfile = self.outf
@@ -3898,7 +3903,7 @@
                     branch.repository, revision_id, time.time(),
                     osutils.local_time_offset(), submit_branch,
                     public_branch=public_branch, include_patch=not no_patch,
-                    include_bundle=not no_bundle, message=None,
+                    include_bundle=not no_bundle, message=message,
                     base_revision_id=base_revision_id)
             elif format == '0.9':
                 if not no_bundle:
@@ -3916,11 +3921,17 @@
                     branch.repository, revision_id, time.time(),
                     osutils.local_time_offset(), submit_branch,
                     public_branch=public_branch, patch_type=patch_type,
-                    message=None)
+                    message=message)
 
             outfile.writelines(directive.to_lines())
             if output is None:
-                branch.get_config().get_mail_client().compose(None, None,
+                subject = '[MERGE] '
+                if message is not None:
+                    subject += message
+                else:
+                    revision = branch.repository.get_revision(revision_id)
+                    subject += revision.message
+                branch.get_config().get_mail_client().compose(mail_to, subject,
                     outfile.getvalue())
         finally:
             if output != '-':
@@ -3961,6 +3972,26 @@
     format 1.  It is compatible with Bazaar 0.12 - 0.18.
     """
 
+    takes_options = [
+        Option('no-bundle',
+               help='Do not include a bundle in the merge directive.'),
+        Option('no-patch', help='Do not include a preview patch in the merge'
+               ' directive.'),
+        Option('remember',
+               help='Remember submit and public branch.'),
+        Option('from',
+               help='Branch to generate the submission from, '
+               'rather than the one containing the working directory.',
+               short_name='f',
+               type=unicode),
+        Option('output', short_name='o', help='Write directive to this file.',
+               type=unicode),
+        'revision',
+        RegistryOption.from_kwargs('format',
+        'Use the specified output format.',
+        **{'4': 'Bundle format 4, Merge Directive 2 (default)',
+           '0.9': 'Bundle format 0.9, Merge Directive 1',})
+        ]
     aliases = ['bundle']
 
     _see_also = ['send', 'merge']

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2007-08-09 19:30:31 +0000
+++ bzrlib/config.py	2007-08-09 20:30:00 +0000
@@ -151,7 +151,7 @@
             None : mail_client.Editor,
             'editor': mail_client.Editor,
             'thunderbird': mail_client.Thunderbird,
-        }[selected_client]()
+        }[selected_client](self)
 
     def _get_signature_checking(self):
         """Template method to override signature checking policy."""

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-07-20 18:59:29 +0000
+++ bzrlib/errors.py	2007-08-09 20:30:00 +0000
@@ -2308,3 +2308,8 @@
 
     def __init__(self, error):
         self.error = error
+
+
+class NoMessageSupplied(BzrError):
+
+    _fmt = "No message supplied."

=== modified file 'bzrlib/mail_client.py'
--- bzrlib/mail_client.py	2007-08-09 19:30:31 +0000
+++ bzrlib/mail_client.py	2007-08-09 20:30:00 +0000
@@ -18,18 +18,38 @@
 import subprocess
 import tempfile
 
-from bzrlib import urlutils
+from bzrlib import (
+    email_message,
+    errors,
+    msgeditor,
+    urlutils,
+    )
 
 
 class MailClient(object):
 
+    def __init__(self, config):
+        self.config = config
+
     def compose(self, to, subject, attachment):
         raise NotImplementedError
 
 
 class Editor(MailClient):
 
-    pass
+    def compose(self, to, subject, attachment):
+        info = ("Please describe these changes:\n\nTo: %s\nSubject: %s\n\n%s"
+                % (to, subject, attachment))
+        body = msgeditor.edit_commit_message(info)
+        if body == '':
+            raise errors.NoMessageSupplied()
+        email_message.EmailMessage.send(self.config,
+                                        self.config.username(),
+                                        to,
+                                        subject,
+                                        body,
+                                        attachment,
+                                        attachment_mime_subtype='x-patch')
 
 
 class Thunderbird(MailClient):

=== modified file 'bzrlib/tests/blackbox/test_send.py'
--- bzrlib/tests/blackbox/test_send.py	2007-08-09 04:54:42 +0000
+++ bzrlib/tests/blackbox/test_send.py	2007-08-09 20:30:00 +0000
@@ -207,3 +207,11 @@
         self.assertIs(merge_directive.MergeDirective, md.__class__)
         self.run_bzr_error(['Bad value .* for option .format.'],
                             'send -f branch -o- --format=0.999')[0]
+
+    def test_message_option(self):
+        self.make_trees()
+        self.run_bzr('send', retcode=3)
+        md = self.send_directive(['--from', 'branch'])
+        self.assertIs(None, md.message)
+        md = self.send_directive(['--from', 'branch', '-m', 'my message'])
+        self.assertEqual('my message', md.message)

=== modified file 'bzrlib/tests/test_mail_client.py'
--- bzrlib/tests/test_mail_client.py	2007-08-09 19:30:31 +0000
+++ bzrlib/tests/test_mail_client.py	2007-08-09 20:30:00 +0000
@@ -24,7 +24,7 @@
 class TestThunderbird(tests.TestCase):
 
     def test_commandline(self):
-        tbird = mail_client.Thunderbird()
+        tbird = mail_client.Thunderbird(None)
         commandline = tbird._get_compose_commandline(None, None,
                                                      'file%')
         self.assertEqual(['-compose', "attachment='%s'" %

